@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  **********************************************************************
  *   Copyright (c) 2001-2011, International Business Machines
@@ -23,7 +25,6 @@
 U_NAMESPACE_BEGIN
 
 class UnicodeMatcher;
-class UnicodeSet;
 
 class U_COMMON_API ICU_Utility /* not : public UObject because all methods are static */ {
  public:
@@ -45,6 +46,13 @@ class U_COMMON_API ICU_Utility /* not : public UObject because all methods are s
                                        int32_t radix = 10,
                                        int32_t minDigits = 1);
 
+    /** Returns a bogus UnicodeString by value. */
+    static inline UnicodeString makeBogusString() {
+        UnicodeString result;
+        result.setToBogus();
+        return result;
+    }
+
     /**
      * Return true if the character is NOT printable ASCII.
      *
@@ -171,12 +179,21 @@ class U_COMMON_API ICU_Utility /* not : public UObject because all methods are s
      * Parse an integer at pos, either of the form \d+ or of the form
      * 0x[0-9A-Fa-f]+ or 0[0-7]+, that is, in standard decimal, hex,
      * or octal format.
-     * @param pos INPUT-OUTPUT parameter.  On input, the first
-     * character to parse.  On output, the character after the last
-     * parsed character.
+     * @param pos INPUT-OUTPUT parameter.  On input, the index of the first
+     * character to parse.  On output, the index of the character after the
+     * last parsed character.
      */
     static int32_t parseInteger(const UnicodeString& rule, int32_t& pos, int32_t limit);
 
+    /**
+     * Parse an integer at pos using only ASCII digits.
+     * Base 10 only.
+     * @param pos INPUT-OUTPUT parameter.  On input, the index of the first
+     * character to parse.  On output, the index of the character after the
+     * last parsed character.
+     */
+    static int32_t parseAsciiInteger(const UnicodeString& str, int32_t& pos);
+
     /**
      * Parse a Unicode identifier from the given string at the given
      * position.  Return the identifier, or an empty string if there
@@ -236,15 +253,5 @@ private:
 
 U_NAMESPACE_END
 
-/**
- * Returns a new set with the Pattern_White_Space characters.
- * The caller must close/delete the result.
- * Stable set of characters, won't change.
- * See UAX #31 Identifier and Pattern Syntax: http://www.unicode.org/reports/tr31/
- * @internal
- */
-U_CAPI U_NAMESPACE_QUALIFIER UnicodeSet* U_EXPORT2
-uprv_openPatternWhiteSpaceSet(UErrorCode* ec);
-
 #endif
 //eof
