@@ -1,8 +1,12 @@
-/********************************************************************
+/*************************************************************************
+ *   © 2016 and later: Unicode, Inc. and others.
+ *   License & terms of use: http://www.unicode.org/copyright.html#License
+ *************************************************************************
+ *************************************************************************
  * COPYRIGHT:
  * Copyright (c) 1999-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
- ********************************************************************/
+ *************************************************************************/
 
 #include "unicode/unistr.h"
 #include "unicode/calendar.h"
@@ -11,6 +15,8 @@
 #include <stdlib.h>
 #include "util.h"
 
+using namespace icu;
+
 /**
  * If the ID supplied to TimeZone is not a valid system ID,
  * TimeZone::createTimeZone() will return a GMT zone object.  In order
@@ -42,13 +48,13 @@ int main(int argc, char **argv) {
     UDate date;
 
     // The languages in which we will display the date
-    static char* LANGUAGE[] = {
+    static const char* LANGUAGE[] = {
         "en", "de", "fr"
     };
     static const int32_t N_LANGUAGE = sizeof(LANGUAGE)/sizeof(LANGUAGE[0]);
 
     // The time zones in which we will display the time
-    static char* TIMEZONE[] = {
+    static const char* TIMEZONE[] = {
         "America/Los_Angeles",
         "America/New_York",
         "Europe/Paris",
