@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2007,2011,2013, International Business Machines
+ * Copyright (c) 1997-2016, International Business Machines
  * Corporation and others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -26,6 +28,7 @@ void addCalTest(TestNode**);
 void addDateForTest(TestNode**);
 void addDateTimePatternGeneratorTest(TestNode**);
 void addDateIntervalFormatTest(TestNode**);
+void addRelativeDateFormatTest(TestNode**);
 void addNumForTest(TestNode**);
 void addMsgForTest(TestNode**);
 void addDateForRgrTest(TestNode**);
@@ -35,6 +38,9 @@ void addUtmsTest(TestNode**);
 void addCurrencyTest(TestNode**);
 void addPluralRulesTest(TestNode**);
 void addURegionTest(TestNode** root);
+void addUListFmtTest(TestNode** root);
+void addUNumberFormatterTest(TestNode** root);
+void addUFormattedValueTest(TestNode** root);
 
 void addFormatTest(TestNode** root);
 
@@ -44,6 +50,9 @@ void addFormatTest(TestNode** root)
     addDateForTest(root);
     addDateTimePatternGeneratorTest(root);
     addDateIntervalFormatTest(root);
+#if !UCONFIG_NO_BREAK_ITERATION
+    addRelativeDateFormatTest(root);
+#endif /* !UCONFIG_NO_BREAK_ITERATION */
     addNumForTest(root);
     addNumFrDepTest(root);
     addMsgForTest(root);
@@ -53,6 +62,9 @@ void addFormatTest(TestNode** root)
     addCurrencyTest(root);
     addPluralRulesTest(root);
     addURegionTest(root);
+    addUListFmtTest(root);
+    addUNumberFormatterTest(root);
+    addUFormattedValueTest(root);
 }
 /*Internal functions used*/
 
