@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 2003-2015, International Business Machines Corporation and
@@ -40,17 +42,20 @@ addHeapMutexTest(TestNode** root)
 
 static int32_t gMutexFailures = 0;
 
-#define TEST_STATUS(status, expected) \
-if (status != expected) { \
-log_err_status(status, "FAIL at  %s:%d. Actual status = \"%s\";  Expected status = \"%s\"\n", \
-  __FILE__, __LINE__, u_errorName(status), u_errorName(expected)); gMutexFailures++; }
+#define TEST_STATUS(status, expected) UPRV_BLOCK_MACRO_BEGIN { \
+    if (status != expected) { \
+        log_err_status(status, "FAIL at  %s:%d. Actual status = \"%s\";  Expected status = \"%s\"\n", \
+                       __FILE__, __LINE__, u_errorName(status), u_errorName(expected)); gMutexFailures++; \
+    } \
+} UPRV_BLOCK_MACRO_END
 
 
-#define TEST_ASSERT(expr) \
-if (!(expr)) { \
-    log_err("FAILED Assertion \"" #expr "\" at  %s:%d.\n", __FILE__, __LINE__); \
-    gMutexFailures++; \
-}
+#define TEST_ASSERT(expr) UPRV_BLOCK_MACRO_BEGIN { \
+    if (!(expr)) { \
+        log_err("FAILED Assertion \"" #expr "\" at  %s:%d.\n", __FILE__, __LINE__); \
+        gMutexFailures++; \
+    } \
+} UPRV_BLOCK_MACRO_END
 
 
 /*  These tests do cleanup and reinitialize ICU in the course of their operation.
@@ -81,6 +86,7 @@ int    gBlockCount = 0;
 const void  *gContext;
 
 static void * U_CALLCONV myMemAlloc(const void *context, size_t size) {
+    (void)context; // suppress compiler warnings about unused variable
     char *retPtr = (char *)malloc(size+sizeof(ctest_AlignedMemory));
     if (retPtr != NULL) {
         retPtr += sizeof(ctest_AlignedMemory);
@@ -90,6 +96,7 @@ static void * U_CALLCONV myMemAlloc(const void *context, size_t size) {
 }
 
 static void U_CALLCONV myMemFree(const void *context, void *mem) {
+    (void)context; // suppress compiler warnings about unused variable
     char *freePtr = (char *)mem;
     if (freePtr != NULL) {
         freePtr -= sizeof(ctest_AlignedMemory);
@@ -100,6 +107,7 @@ static void U_CALLCONV myMemFree(const void *context, void *mem) {
 
 
 static void * U_CALLCONV myMemRealloc(const void *context, void *mem, size_t size) {
+    (void)context; // suppress compiler warnings about unused variable
     char *p = (char *)mem;
     char *retPtr;
 
