@@ -31,6 +31,11 @@ private:
     void testStaticGetSkeleton(/* char* par */);
     void testC();
     void testSkeletonsWithDayPeriods();
+    void testGetFieldDisplayNames();
+    void testJjMapping();
+    void test20640_HourCyclArsEnNH();
+    void testFallbackWithDefaultRootLocale();
+    void testGetDefaultHourCycle_OnEmptyInstance();
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
