@@ -6,6 +6,7 @@
 
 #include <string>
 #include <vector>
+#include <utility>
 
 #include "unicode/utypes.h"
 #include "unicode/localematcher.h"
@@ -58,6 +59,7 @@ public:
     void testSupportedDefault();
     void testUnsupportedDefault();
     void testDemotion();
+    void testDirection();
     void testMatch();
     void testResolvedLocale();
     void testDataDriven();
@@ -81,6 +83,7 @@ void LocaleMatcherTest::runIndexedTest(int32_t index, UBool exec, const char *&n
     TESTCASE_AUTO(testSupportedDefault);
     TESTCASE_AUTO(testUnsupportedDefault);
     TESTCASE_AUTO(testDemotion);
+    TESTCASE_AUTO(testDirection);
     TESTCASE_AUTO(testMatch);
     TESTCASE_AUTO(testResolvedLocale);
     TESTCASE_AUTO(testDataDriven);
@@ -272,7 +275,7 @@ void LocaleMatcherTest::testSupportedDefault() {
     assertEquals("getBestMatchResult(ja_JP).supp",
                  "en_GB", locString(result.getSupportedLocale()));
     assertEquals("getBestMatchResult(ja_JP).suppIndex",
-                 1, result.getSupportedIndex());
+                 -1, result.getSupportedIndex());
 }
 
 void LocaleMatcherTest::testUnsupportedDefault() {
@@ -322,6 +325,36 @@ void LocaleMatcherTest::testDemotion() {
     }
 }
 
+void LocaleMatcherTest::testDirection() {
+    IcuTestErrorCode errorCode(*this, "testDirection");
+    Locale supported[] = { "ar", "nn" };
+    Locale desired[] = { "arz-EG", "nb-DK" };
+    LocaleMatcher::Builder builder;
+    builder.setSupportedLocales(ARRAY_RANGE(supported));
+    {
+        // arz is a close one-way match to ar, and the region matches.
+        // (Egyptian Arabic vs. Arabic)
+        // Also explicitly exercise the move copy constructor.
+        LocaleMatcher built = builder.build(errorCode);
+        LocaleMatcher withOneWay(std::move(built));
+        Locale::RangeIterator<Locale *> desiredIter(ARRAY_RANGE(desired));
+        assertEquals("with one-way", "ar",
+                     locString(withOneWay.getBestMatch(desiredIter, errorCode)));
+    }
+    {
+        // nb is a less close two-way match to nn, and the regions differ.
+        // (Norwegian Bokmal vs. Nynorsk)
+        // Also explicitly exercise the move assignment operator.
+        LocaleMatcher onlyTwoWay = builder.build(errorCode);
+        LocaleMatcher built =
+            builder.setDirection(ULOCMATCH_DIRECTION_ONLY_TWO_WAY).build(errorCode);
+        onlyTwoWay = std::move(built);
+        Locale::RangeIterator<Locale *> desiredIter(ARRAY_RANGE(desired));
+        assertEquals("only two-way", "nn",
+                     locString(onlyTwoWay.getBestMatch(desiredIter, errorCode)));
+    }
+}
+
 void LocaleMatcherTest::testMatch() {
     IcuTestErrorCode errorCode(*this, "testMatch");
     LocaleMatcher matcher = LocaleMatcher::Builder().build(errorCode);
