@@ -294,11 +294,13 @@ class NumberFormatTest: public CalendarTimeZoneTest {
     void Test13840_ParseLongStringCrash();
     void Test13850_EmptyStringCurrency();
     void Test20348_CurrencyPrefixOverride();
+    void Test20956_MonetarySymbolGetters();
     void Test20358_GroupingInPattern();
     void Test13731_DefaultCurrency();
     void Test20499_CurrencyVisibleDigitsPlural();
     void Test13735_GroupingSizeGetter();
     void Test13734_StrictFlexibleWhitespace();
+    void Test20961_CurrencyPluralPattern();
 
  private:
     UBool testFormattableAsUFormattable(const char *file, int line, Formattable &f);
