@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2005, International Business Machines Corporation and
+ * Copyright (c) 1997-2011, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -30,7 +32,6 @@ private:
     void testPositiveInfinity(void);
     void testNegativeInfinity(void);
     void testZero(void);
-    void testU_INLINE();
 
     // subtests of testNaN
     void testIsNaN(void);
