@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007-2009, International Business Machines Corporation and    *
+* Copyright (C) 2007-2011, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 */
@@ -36,6 +38,8 @@ public:
     void TestVTimeZoneParse(void);
     void TestT6216(void);
     void TestT6669(void);
+    void TestVTimeZoneWrapper(void);
+    void TestT8943(void);
 
 private:
     void verifyTransitions(BasicTimeZone& icutz, UDate start, UDate end);
