@@ -1,12 +1,14 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2003-2009, International Business Machines
+*   Copyright (C) 2003-2015, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
 *   file name:  pkgitems.cpp
-*   encoding:   US-ASCII
+*   encoding:   UTF-8
 *   tab size:   8 (not used)
 *   indentation:4
 *
@@ -43,8 +45,6 @@
 
 // general definitions ----------------------------------------------------- ***
 
-#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
-
 U_CDECL_BEGIN
 
 static void U_CALLCONV
@@ -120,8 +120,6 @@ private:
     int32_t length;
 };
 
-U_NAMESPACE_END
-
 // check a dependency ------------------------------------------------------ ***
 
 /*
@@ -284,6 +282,7 @@ ures_enumDependencies(const char *itemName,
                       const ResourceData *pResData,
                       Resource res, const char *inKey, const char *parentKey, int32_t depth,
                       CheckDependency check, void *context,
+                      Package *pkg,
                       UErrorCode *pErrorCode) {
     switch(res_getPublicType(res)) {
     case URES_STRING:
@@ -306,7 +305,8 @@ ures_enumDependencies(const char *itemName,
                 break;
             }
             int32_t length;
-            const UChar *alias=res_getString(pResData, res, &length);
+            // No tracing: build tool
+            const UChar *alias=res_getStringNoTrace(pResData, res, &length);
             checkAlias(itemName, res, alias, length, useResSuffix, check, context, pErrorCode);
         }
         break;
@@ -329,6 +329,7 @@ ures_enumDependencies(const char *itemName,
                         item, itemKey,
                         inKey, depth+1,
                         check, context,
+                        pkg,
                         pErrorCode);
                 if(U_FAILURE(*pErrorCode)) {
                     fprintf(stderr, "icupkg/ures_enumDependencies(%s table res=%08x)[%d].recurse(%s: %08x) failed\n",
@@ -349,6 +350,7 @@ ures_enumDependencies(const char *itemName,
                         item, NULL,
                         inKey, depth+1,
                         check, context,
+                        pkg,
                         pErrorCode);
                 if(U_FAILURE(*pErrorCode)) {
                     fprintf(stderr, "icupkg/ures_enumDependencies(%s array res=%08x)[%d].recurse(%08x) failed\n",
@@ -367,6 +369,7 @@ static void
 ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
                       const uint8_t *inBytes, int32_t length,
                       CheckDependency check, void *context,
+                      Package *pkg,
                       UErrorCode *pErrorCode) {
     ResourceData resData;
 
@@ -388,7 +391,7 @@ ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
         }
     }
 
-    U_NAMESPACE_QUALIFIER NativeItem nativePool;
+    icu::NativeItem nativePool;
 
     if(resData.usesPoolBundle) {
         char poolName[200];
@@ -397,9 +400,6 @@ ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
             return;
         }
         check(context, itemName, poolName);
-        // TODO: The Package should be passed in.
-        // Since the context is always a Package, we could just redeclare it.
-        U_NAMESPACE_QUALIFIER Package *pkg=(U_NAMESPACE_QUALIFIER Package *)context;
         int32_t index=pkg->findItem(poolName);
         if(index<0) {
             // We cannot work with a bundle if its pool resource is missing.
@@ -413,7 +413,8 @@ ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
             fprintf(stderr, "icupkg: %s is not a pool bundle\n", poolName);
             return;
         }
-        const int32_t *poolIndexes=(const int32_t *)nativePool.getBytes()+1;
+        const int32_t *poolRoot=(const int32_t *)nativePool.getBytes();
+        const int32_t *poolIndexes=poolRoot+1;
         int32_t poolIndexLength=poolIndexes[URES_INDEX_LENGTH]&0xff;
         if(!(poolIndexLength>URES_INDEX_POOL_CHECKSUM &&
              (poolIndexes[URES_INDEX_ATTRIBUTES]&URES_ATT_IS_POOL_BUNDLE))
@@ -423,6 +424,7 @@ ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
         }
         if(resData.pRoot[1+URES_INDEX_POOL_CHECKSUM]==poolIndexes[URES_INDEX_POOL_CHECKSUM]) {
             resData.poolBundleKeys=(const char *)(poolIndexes+poolIndexLength);
+            resData.poolBundleStrings=(const uint16_t *)(poolRoot+poolIndexes[URES_INDEX_KEYS_TOP]);
         } else {
             fprintf(stderr, "icupkg: %s has mismatched checksum for %s\n", poolName, itemName);
             return;
@@ -433,6 +435,7 @@ ures_enumDependencies(const char *itemName, const UDataInfo *pInfo,
         itemName, &resData,
         resData.rootRes, NULL, NULL, 0,
         check, context,
+        pkg,
         pErrorCode);
 }
 
@@ -572,8 +575,6 @@ getDataFormat(const uint8_t dataFormat[4]) {
 
 // enumerate dependencies of a package item -------------------------------- ***
 
-U_NAMESPACE_BEGIN
-
 void
 Package::enumDependencies(Item *pItem, void *context, CheckDependency check) {
     int32_t infoLength, itemHeaderLength;
@@ -595,7 +596,7 @@ Package::enumDependencies(Item *pItem, void *context, CheckDependency check) {
                  * We do not want to duplicate that code, especially not together with on-the-fly swapping.
                  */
                 NativeItem nrb(pItem, ures_swap);
-                ures_enumDependencies(pItem->name, nrb.getDataInfo(), nrb.getBytes(), nrb.getLength(), check, context, &errorCode);
+                ures_enumDependencies(pItem->name, nrb.getDataInfo(), nrb.getBytes(), nrb.getLength(), check, context, this, &errorCode);
                 break;
             }
         case FMT_CNV:
